(*
 * Copyright (c) Facebook, Inc. and its affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the "hack" directory of this source tree.
 *
 *)

(** A generic exception for all refactor sound dynamic specific failures
   Relationship with shape_analysis: Shape_analysis_exn *)
exception Refactor_sd_exn of string

type mode =
  | FlagTargets
      (** Flag all possible targets without performing any analysis *)
  | DumpConstraints  (** Dump constraints generated by analysing the program *)
  | SimplifyConstraints
      (** Partially solve key constraints within functions and methods and
          report back whether a function is ever upcasted to dynamic. *)
  | SolveConstraints
      (** Globally solve the key constraints and report back whether a
          function is ever upcasted to dynamic *)

type options = { mode: mode }

(** Relationship with shape_analysis: shape_result *)
type refactor_sd_result =
  | Exists_Upcast
  | No_Upcast
